package cn.iocoder.yudao.module.system.handler.mapnode.strategy;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.framework.common.util.collection.CollectionUtils;
import cn.iocoder.yudao.module.system.controller.admin.positionmap.dto.NodeBaseDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.service.houselocation.HouseLocationService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

import static com.baomidou.mybatisplus.core.toolkit.IdWorker.getId;

@Component
public class HouseLocationStrategyImpl implements NodeProcessingStrategy {
    @Resource
    private HouseLocationService houseLocationService;

    @Override
    public void processNodes(Long positionMapId, List<NodeBaseDTO> nodeBaseDTOS) {
        // -- 策略1 处理库位点
        // -- 将data里面的json 数据转为实体类 - 再对比节点id - 然后做新增删除修改操作
        List<WareHouseLocationDO> newList = new ArrayList<>();
        nodeBaseDTOS.forEach(item -> {
            if (item.getId() == null) {
                item.setId(getId());
            }
            item.setPositionMapId(positionMapId);
            // -- 如果是库位点 - 可能是一对多情况 - 需要将多个库位点的id set进去
            List<WareHouseLocationDO> wareHouseLocationDOS = JSONUtil.toList(item.getDataJson(), WareHouseLocationDO.class);
            wareHouseLocationDOS.forEach(wareHouseLocationDO -> {
                wareHouseLocationDO.setLocationX(item.getLocationX());
                wareHouseLocationDO.setLocationY(item.getLocationY());
                if (wareHouseLocationDO.getId() == null) {
                    wareHouseLocationDO.setId(getId());
                }
                wareHouseLocationDO.setMapId(positionMapId);
                wareHouseLocationDO.setMapItemId(item.getId());
                newList.add(wareHouseLocationDO);
            });
            item.setDataJson(JSONUtil.toJsonStr(wareHouseLocationDOS));
        });

        List<WareHouseLocationDO> oldList = houseLocationService.getByMapId(positionMapId);
        List<List<WareHouseLocationDO>> list = CollectionUtils.diffList(oldList, newList,
                (oldVal, newVal) -> ObjectUtil.equal(oldVal.getId(), newVal.getId()));
        houseLocationService.batchSaveOrEditOrDel(positionMapId, list);
    }
}
