package cn.iocoder.yudao.module.system.service.information;

import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.grpc.api.GrpcServiceApi;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.information.DeviceInformationDO;
import cn.iocoder.yudao.module.system.dal.mysql.information.DeviceInformationMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

import static cn.hutool.core.collection.CollUtil.isNotEmpty;
import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertList;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.INFORMATION_MAC_EXIST;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.INFORMATION_NOT_EXISTS;

/**
 * 设备信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
@Slf4j
public class DeviceInformationServiceImpl implements DeviceInformationService {

    @Resource
    private DeviceInformationMapper informationMapper;

    @Resource
    private GrpcServiceApi grpcServiceApi;

    @Override
    public Long createInformation(DeviceInformationSaveReqVO createReqVO) {
        DeviceInformationPageReqVO pageReqVO = new DeviceInformationPageReqVO();
        pageReqVO.setMacAddress(createReqVO.getMacAddress());
        PageResult<DeviceInformationDO> pageResult = informationMapper.selectPage(pageReqVO);
        if (ObjectUtil.isNotEmpty(pageResult.getList())) {
            throw exception(INFORMATION_MAC_EXIST);
        }
        // 插入
        DeviceInformationDO information = BeanUtils.toBean(createReqVO, DeviceInformationDO.class);
        information.setDeviceStatus(1);
        information.setDeviceLastTime(LocalDateTime.now());
        informationMapper.insert(information);
        // 返回
        return information.getId();
    }

    @Override
    public void updateInformation(DeviceInformationSaveReqVO updateReqVO) {
        // 校验存在
        validateInformationExists(updateReqVO.getId());
        // 更新
        DeviceInformationDO updateObj = BeanUtils.toBean(updateReqVO, DeviceInformationDO.class);
        informationMapper.updateById(updateObj);
    }

    @Override
    public void deleteInformation(Long id) {
        // 校验存在
        validateInformationExists(id);
        // 删除
        informationMapper.deleteById(id);
    }

    private void validateInformationExists(Long id) {
        if (informationMapper.selectById(id) == null) {
            throw exception(INFORMATION_NOT_EXISTS);
        }
    }

    @Override
    public DeviceInformationDO getInformation(Long id) {
        return informationMapper.selectById(id);
    }

    @Override
    public PageResult<DeviceInformationDO> getInformationPage(DeviceInformationPageReqVO pageReqVO) {
        return informationMapper.selectPage(pageReqVO);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchSaveOrEditOrDel(Long positionMapId, List<List<DeviceInformationDO>> list) {
        //批量添加、修改、删除
        if (isNotEmpty(list.get(0))) {
            informationMapper.insertBatch(list.get(0));
        }
        if (isNotEmpty(list.get(1))) {
            informationMapper.updateBatch(list.get(1));
        }
        if (isNotEmpty(list.get(2))) {
            informationMapper.deleteByIds(convertList(list.get(2), DeviceInformationDO::getId));
        }
    }

    @Override
    public List<DeviceInformationDO> getByMapId(Long positionMapId) {
        return informationMapper.selectList(new LambdaQueryWrapperX<DeviceInformationDO>()
                .eq(DeviceInformationDO::getPositionMapId, positionMapId));
    }

    @Override
    public List<DeviceInformationDO> getList(DeviceInformationPageReqVO pageReqVO) {
        return informationMapper.selectList(new LambdaQueryWrapper<DeviceInformationDO>()
                .eq(pageReqVO.getDeviceType() != null, DeviceInformationDO::getDeviceType, pageReqVO.getDeviceType()));
    }

}
