package cn.iocoder.yudao.module.system.service.robot;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import cn.iocoder.yudao.module.system.api.robot.dto.RobotStatusDataPoseDTO;
import cn.iocoder.yudao.module.system.constant.robot.RobotTaskChcheConstant;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.*;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotInformationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.enums.robot.*;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.v3.oas.annotations.media.Schema;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 车辆信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RobotInformationServiceImpl implements RobotInformationService {

    @Resource
    private RobotInformationMapper informationMapper;

    @Resource
    private PositionMapMapper positionMapMapper;

    @Resource
    private RobotTaskDetailMapper taskDetailMapper;

    @Resource
    private RedisUtil redisUtil;

    @Override
    public Long createInformation(RobotInformationSaveReqVO createReqVO) {
        //判断mac地址是否重复
        RobotInformationDO query = new RobotInformationDO();
        query.setMacAddress(createReqVO.getMacAddress());
        List<RobotInformationDO> existRobotMac = informationMapper.queryAllByLimit(query);
        if (ObjectUtil.isNotEmpty(existRobotMac)) {
            throw exception(ROBOT_MAC_ADDRESS_EXISTS);
        }
        //判断机器人编号
        query.setMacAddress(null);
        query.setRobotNo(createReqVO.getRobotNo());
        List<RobotInformationDO> existRobotNo = informationMapper.queryAllByLimit(query);
        if (ObjectUtil.isNotEmpty(existRobotNo)) {
            throw exception(ROBOT_ROBOT_NO_EXISTS);
        }
        // 插入
        RobotInformationDO information = BeanUtils.toBean(createReqVO, RobotInformationDO.class);
        informationMapper.insert(information);
        // 返回
        return information.getId();
    }

    @Override
    public void updateInformation(RobotInformationSaveReqVO updateReqVO) {
        // 校验存在
        validateInformationExists(updateReqVO.getId());
        //校验MAC地址
        RobotInformationDO query = new RobotInformationDO();
        query.setMacAddress(updateReqVO.getMacAddress());
        List<RobotInformationDO> existRobot = informationMapper.queryAllByLimit(query);
        boolean exist = existRobot.stream().anyMatch(v -> (v.getMacAddress().equals(updateReqVO.getMacAddress())
                && !v.getId().equals(updateReqVO.getId())));
        if (exist) {
            throw exception(ROBOT_MAC_ADDRESS_EXISTS);
        }
        //校验机器人编号
        query.setMacAddress(null);
        query.setRobotNo(updateReqVO.getRobotNo());
        List<RobotInformationDO> existRobotNo = informationMapper.queryAllByLimit(query);
        boolean robotNoExist = existRobotNo.stream().anyMatch(v -> (v.getRobotNo().equals(updateReqVO.getRobotNo())
                && !v.getId().equals(updateReqVO.getId())));
        if (robotNoExist) {
            throw exception(ROBOT_ROBOT_NO_EXISTS);
        }
        // 更新
        RobotInformationDO updateObj = BeanUtils.toBean(updateReqVO, RobotInformationDO.class);
        informationMapper.updateById(updateObj);
    }

    @Override
    public void deleteInformation(Long id) {
        // 校验存在
        validateInformationExists(id);
        // 删除
        informationMapper.deleteById(id);
    }

    private void validateInformationExists(Long id) {
        if (informationMapper.selectById(id) == null) {
            throw exception(ROBOT_INFORMATION_NOT_EXISTS);
        }
    }

    @Override
    public RobotInformationRespVO getInformation(Long id) {
        RobotInformationDO robotInformationDO = informationMapper.selectById(id);
        RobotInformationRespVO bean = BeanUtils.toBean(robotInformationDO, RobotInformationRespVO.class);
        setPositionMapList(bean);
        return bean;
    }

    public void setPositionMapList(RobotInformationRespVO bean) {
        Set<Long> ids = bean.getFloorAreaJson();
        if (ObjectUtil.isNotEmpty(ids)) {
            List<PositionMapDO> list = positionMapMapper.selectByIds(ids);
            List<RobotPositionMapVO> targetList = BeanUtil.copyToList(list, RobotPositionMapVO.class);
            bean.setPositionMapList(targetList);
        }
    }

    @Override
    public PageResult<RobotInformationPageRespVO> getInformationPage(RobotInformationPageReqVO pageReqVO) {
        PageResult<RobotInformationDO> pageResult = informationMapper.selectPage(pageReqVO);

        PageResult<RobotInformationPageRespVO> dataPage = new PageResult<>();
        dataPage.setTotal(pageResult.getTotal());
        List<RobotInformationDO> list = pageResult.getList();
        List<RobotInformationPageRespVO> targetList = BeanUtil.copyToList(list, RobotInformationPageRespVO.class);
        targetList.stream().forEach(v -> {
            setMsgAndRobotStatus(v);
        });
        dataPage.setList(targetList);

        return dataPage;
    }

    /**
     * 设置状态和信息
     * @param v
     */
    private void setMsgAndRobotStatus(RobotInformationPageRespVO v) {
        String pose2dKey = RobotTaskChcheConstant.ROBOT_POSE_BAT +v.getMacAddress();
        Object object = redisUtil.get(pose2dKey);
        RobotStatusDataPoseDTO robotStatusDataPoseDTO= JSONUtil.toBean((String)object, RobotStatusDataPoseDTO.class);

        String robotDoingActionKey = RobotTaskChcheConstant.ROBOT_DOING_ACTION +v.getMacAddress();
        Object action = redisUtil.get(robotDoingActionKey);

        if (ObjectUtil.isNotEmpty(object) && ObjectUtil.isNotEmpty(robotStatusDataPoseDTO)) {
            v.setElectricity(robotStatusDataPoseDTO.getBat_soc());
            v.setFloor(robotStatusDataPoseDTO.getFloor());
            v.setArea(robotStatusDataPoseDTO.getArea());
        }

        if (ObjectUtil.isEmpty(object)) {
            v.setRobotStatus(3);
            v.setMsg("车辆已经离线");
        }else if (RobotTaskModelEnum.REJECTION.getType().equals(v.getRobotTaskModel())) {
            v.setRobotStatus(2);
            v.setMsg("车辆已经锁定");
        }else if (RobotStatusEnum.STAND_BY.getType().equals(v.getRobotStatus())) {
            //查看机器人最后做的任务是不是充电
            RobotTaskDetailDO robotTaskDetailDO = taskDetailMapper.selectOne(new LambdaQueryWrapper<RobotTaskDetailDO>()
                    .eq(RobotTaskDetailDO::getRobotNo, v.getRobotNo())
                    .orderByDesc(RobotTaskDetailDO::getCreateTime)
                    .last("limit 1"));
            if (ObjectUtil.isNotEmpty(robotTaskDetailDO)
                    && RobotTaskTypeEnum.CHARGE.getType().equals(robotTaskDetailDO.getTaskType())
                    && RobotTaskStatusEnum.DOING.getType().equals(robotTaskDetailDO.getTaskStatus())) {
                v.setRobotStatus(4);
                v.setMsg("车辆正在充电");
            }else {
                v.setRobotStatus(0);
                v.setMsg("车辆正在待命");
            }
        }else if (RobotStatusEnum.DOING.getType().equals(v.getRobotStatus()) && ObjectUtil.isNotEmpty(action)) {
            v.setRobotStatus(1);
            CommandTypeEnum commandType = CommandTypeEnum.getCommandType(String.valueOf(action));
            if (ObjectUtil.isNotEmpty(commandType)) {
                v.setMsg("车辆正在"+commandType.getMsg());
            }
        } else {
            v.setRobotStatus(5);
            v.setMsg("车辆发生异常");
        }

    }

    /**
     * 统计车辆待命/任务中/离线
     * @return
     */
    @Override
    public RobotInformationStatisticsVO statisticsInformation() {
        RobotInformationStatisticsVO info = new RobotInformationStatisticsVO();

        RobotInformationDO query = new RobotInformationDO();
        List<RobotInformationDO> existRobot = informationMapper.queryAllByLimit(query);
        if (ObjectUtil.isEmpty(existRobot)) {
            return info;
        }
        Integer standby = 0;
        Integer inTask = 0;
        Integer doLock = 0;
        Integer offline = 0;
        Integer charge = 0;
        Integer fault = 0;
        //0：待命、1：任务中、2：锁定、3：离线、4:充电中、5:故障
        for (RobotInformationDO v : existRobot) {
            String pose2dKey = RobotTaskChcheConstant.ROBOT_POSE_BAT +v.getMacAddress();
            Object object = redisUtil.get(pose2dKey);
            if (ObjectUtil.isEmpty(object)) {
                offline++;
            }else if (RobotTaskModelEnum.REJECTION.getType().equals(v.getRobotTaskModel())) {
                doLock++;
            }else if (RobotStatusEnum.STAND_BY.getType().equals(v.getRobotStatus())) {

                //查看机器人最后做的任务是不是充电
                RobotTaskDetailDO robotTaskDetailDO = taskDetailMapper.selectOne(new LambdaQueryWrapper<RobotTaskDetailDO>()
                        .eq(RobotTaskDetailDO::getRobotNo, v.getRobotNo())
                        .orderByDesc(RobotTaskDetailDO::getCreateTime)
                        .last("limit 1"));
                if (ObjectUtil.isNotEmpty(robotTaskDetailDO)
                        && RobotTaskTypeEnum.CHARGE.getType().equals(robotTaskDetailDO.getTaskType())
                        && RobotTaskStatusEnum.DOING.getType().equals(robotTaskDetailDO.getTaskStatus())) {
                    charge++;
                }else {
                    standby++;
                }

            }else if (RobotStatusEnum.DOING.getType().equals(v.getRobotStatus())) {
                inTask++;
            } else {
                fault++;
            }
        }
        info.setStandby(standby);
        info.setInTask(inTask);
        info.setDoLock(doLock);
        info.setOffline(offline);
        info.setCharge(charge);
        info.setFault(fault);

        return info;
    }

    /**
     *
     * @return
     */
    @Override
    public List<RobotInformationDO> getCanUseRobot() {
        RobotInformationDO query = new RobotInformationDO();
        query.setRobotTaskModel(RobotTaskModelEnum.NORMAL.getType());
        return informationMapper.queryAllByLimit(query);
    }

    @Override
    public List<RobotInformationDO> getAllRobot() {
        RobotInformationDO query = new RobotInformationDO();
        return informationMapper.queryAllByLimit(query);
    }

    /**
     * 根据mac查询机器人编号
     * @param mac
     * @return
     */
    @Override
    public String getRobotNoByMac(String mac) {
        if (ObjectUtil.isEmpty(mac)) {
            return "";
        }
        String robotNo = (String) redisUtil.get(RobotTaskChcheConstant.ROBOT_NO_MAPPING_MAC + mac);

        if (ObjectUtil.isNotEmpty(robotNo)) {
            return robotNo;
        }

        RobotInformationDO robotInformationDO = informationMapper.selectOne(new LambdaQueryWrapper<RobotInformationDO>()
                .eq(RobotInformationDO::getMacAddress, mac)
                .last("limit 1"));
        if (ObjectUtil.isNotEmpty(robotInformationDO)) {
            redisUtil.set(RobotTaskChcheConstant.ROBOT_NO_MAPPING_MAC + mac,robotInformationDO.getRobotNo());
            return robotInformationDO.getRobotNo();
        }
        return "";
    }

}